#include "latfold.h"

enum {
	COL_SCORE,
	COL_TIME,
	N_COLUMNS
};

/** @brief スコアを通過した時間を表示するためのウィジェットを作成する関数
 *  @param[in] gui LatFoldGUIのポインター
 */
GtkWidget *latfold_gui_clist_new(LatFoldGUI *gui) {
	GtkWidget *w;
	GtkTreeModel *model;
	gint i;
	/* create tree model */
	gui->store = gtk_list_store_new(N_COLUMNS,
			G_TYPE_INT,    /* COL_SCORE */
			G_TYPE_STRING  /* COL_TIME */
			);
	/* create tree view */
	model = GTK_TREE_MODEL(gui->store);
	w = gtk_tree_view_new_with_model(model);
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(w), TRUE);
	g_object_unref(model);
	model = NULL;
	for (i = 0; i < N_COLUMNS; ++i) {
		GtkTreeViewColumn *col;
		GtkCellRenderer *renderer;
		GtkWidget *label;
		char *table_label[] = {_("Score"), _("Time")};
		renderer = gtk_cell_renderer_text_new ();
		col = gtk_tree_view_column_new ();
		gtk_tree_view_column_pack_start (col, renderer, TRUE);
		gtk_tree_view_column_add_attribute (col, renderer, "text", i);
		gtk_tree_view_append_column (GTK_TREE_VIEW(w), col);
		label = gtk_label_new(table_label[i]);
		gtk_tree_view_column_set_widget(col, label);
		gtk_tree_view_column_set_alignment(col, 0.5);
		gtk_widget_show(label);
	}
	return w;
}

/** @brief 通過時間を追加するための関数
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[in] score スコア
 *  @param[in] str_time 通過時間（文字列）
 *  @param[in] overwrite 上書きするかどうかのフラグ
 */
void latfold_gui_clist_set_score(LatFoldGUI *gui, gint score, gchar *str_time, gboolean overwrite) {
	GtkTreeIter iter;
	gint i;
	gboolean valid;
	gboolean added = FALSE;
	/* score番目までiterを回す */
	for (i = 1; i <= score; ++i) {
		if (i == 1) {
			valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(gui->store), &iter);
		} else {
			valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(gui->store), &iter);
		}
		/* 存在しないなら新規に追加する */
		if (! valid) {
			gtk_list_store_append(gui->store, &iter);
			gtk_list_store_set(gui->store, &iter,
					COL_SCORE, i,
					COL_TIME, "--:--:--",
					-1);
			added = TRUE;
		}
	}
	if (overwrite || added) {
		gtk_list_store_set(gui->store, &iter, COL_TIME, str_time, -1);
	}
	return;
}

/** @brief 通過時間に関するウィジェット等のデータを破棄する関数
 *  @param[in] gui LatFoldGUIのポインター
 */
void latfold_gui_clist_clear(LatFoldGUI *gui) {
	gtk_list_store_clear(gui->store);
	gui->clist_last_score = 0;
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(gui->progress), 0.0);
	return;
}
