#include "latfold.h"
#include "icon/bend1.xpm"
#include "icon/bend2.xpm"
#include "icon/bend3.xpm"
#include "icon/bend4.xpm"
#include "icon/flip1.xpm"
#include "icon/flip2.xpm"
#include "icon/flip3.xpm"
#include "icon/flip4.xpm"
#include "icon/arrow.xpm"
#include "icon/arrow_mouse.xpm"
#include "icon/score.xpm"

/** @brief ヘルプ用の画像ファイルをロードする関数
 *  @param[in] gui LatFoldGUIのポインター
 */
void load_help_pixmap(LatFoldGUI *gui) {
	static char **file_bend[] = {bend1, bend2, bend3, bend4};
	static char **file_flip[] = {flip1, flip2, flip3, flip4};
	gint i;
	for (i = 0; i < 4; ++i) {
		gui->pixmap_bend[i] = gdk_pixmap_create_from_xpm_d(gui->main->window,
				&gui->mask_bend[i], NULL, file_bend[i]);
		gui->pixmap_flip[i] = gdk_pixmap_create_from_xpm_d(gui->main->window,
				&gui->mask_flip[i], NULL, file_flip[i]);
	}
	gui->pixmap_arrow = gdk_pixmap_create_from_xpm_d(gui->main->window,
				&gui->mask_arrow, NULL, arrow);
	gui->pixmap_mouse = gdk_pixmap_create_from_xpm_d(gui->main->window,
				&gui->mask_mouse, NULL, arrow_mouse);
	gui->pixmap_score = gdk_pixmap_create_from_xpm_d(gui->main->window,
				&gui->mask_score, NULL, score);
	return;
}

/** @brief ヘルプダイアローグを表示する関数
 *  @param[in] lf LatFoldのポインター
 */
void help_dialog(GtkAction *action, LatFold *lf) {
	GtkWidget *dialog, *w, *box, *frame, *hbox;
	LatFoldGUI *gui = lf->gui;
	gchar *str;
	GdkPixmap *pixmap_bend[] = {
		gui->pixmap_bend[0], gui->pixmap_mouse,
		gui->pixmap_bend[1], gui->pixmap_arrow,
		gui->pixmap_bend[2], gui->pixmap_mouse,
		gui->pixmap_bend[3]
	};
	GdkBitmap *mask_bend[] = {
		gui->mask_bend[0], gui->mask_mouse,
		gui->mask_bend[1], gui->mask_arrow,
		gui->mask_bend[2], gui->mask_mouse,
		gui->mask_bend[3]
	};
	GdkPixmap *pixmap_flip[] = {
		gui->pixmap_flip[0], gui->pixmap_mouse,
		gui->pixmap_flip[1], gui->pixmap_arrow,
		gui->pixmap_flip[2], gui->pixmap_mouse,
		gui->pixmap_flip[3]
	};
	GdkBitmap *mask_flip[] = {
		gui->mask_flip[0], gui->mask_mouse,
		gui->mask_flip[1], gui->mask_arrow,
		gui->mask_flip[2], gui->mask_mouse,
		gui->mask_flip[3]
	};
	GdkPixmap *pixmap_score[] = {
		gui->pixmap_score, gui->pixmap_arrow
	};
	GdkBitmap *mask_score[] = {
		gui->mask_score, gui->mask_arrow
	};
	gint i;
	g_return_if_fail(action != NULL);
	/* dialog on toplevel */
	dialog = gtk_dialog_new_with_buttons(_("About instruction and score"),
			GTK_WINDOW(gui->main),
			(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
			GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
			NULL);
	box = GTK_DIALOG(dialog)->vbox;
	/* frame_bend in dialog */
	frame = w = gtk_frame_new(_("How to bend"));
	gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 2);
	gtk_widget_show(w);
	/* hbox_bend in frame */
	hbox = w = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), w);
	gtk_widget_show(w);
	/* pixmaps in hbox_bend */
	for (i = 0; i < 7; ++i) {
		w = gtk_image_new_from_pixmap(pixmap_bend[i], mask_bend[i]);
		gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 2);
		gtk_widget_show(w);
	}
	/* frame_flip in dialog */
	frame = w = gtk_frame_new(_("How to flip"));
	gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 2);
	gtk_widget_show(w);
	/* hbox_flip in frame */
	hbox = w = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), w);
	gtk_widget_show(w);
	/* pixmaps in hbox_flip */
	for (i = 0; i < 7; ++i) {
		w = gtk_image_new_from_pixmap(pixmap_flip[i], mask_flip[i]);
		gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 2);
		gtk_widget_show(w);
	}
	/* frame_score in dialog */
	frame = w = gtk_frame_new(_("How to calculate score"));
	gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 2);
	gtk_widget_show(w);
	/* hbox_score in frame */
	hbox = w = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), w);
	gtk_widget_show(w);
	/* pixmaps in hbox_score */
	for (i = 0; i < 2; ++i) {
		w = gtk_image_new_from_pixmap(pixmap_score[i], mask_score[i]);
		gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 10);
		gtk_widget_show(w);
	}
	/* label in hbox_score */
	str = g_strdup_printf("%s: 9", _("Score"));
	w = gtk_label_new(str);
	g_nullify_pointer((gpointer)&str);
	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 10);
	gtk_widget_show(w);
	switch (gtk_dialog_run(GTK_DIALOG(dialog))) {
		case GTK_RESPONSE_ACCEPT:
		default:
			break;
	}
	gtk_widget_destroy(dialog);
	return;
}

