#include "latfold.h"

static gboolean create_new_dialog(LatFoldGUI *gui, gint *natom, gint *seed);

/** @brief 新規ゲームのダイアローグを開き、分子データを作成する関数
 *  @param[in] lf LatFoldのポインター
 */
void open_new_dialog(LatFold *lf) {
	print_enter();
	if (create_new_dialog(lf->gui, &lf->opt->natom, &lf->opt->seed)) {
		if (lf->data != NULL) {
			latfold_data_destroy(lf->data);
			lf->data = NULL;
		}
		lf->data = latfold_data_new(lf->gui, lf->opt->natom, lf->opt->seed);
		lf->opt->natom = lf->data->natom;
		lf->opt->seed = lf->data->seed;
		if (lf->data->solve != NULL) {
			solve_destroy(lf->data);
		}
		solve_init(lf->data);
		/* アイドリング時に呼ぶ関数を登録する */
		lf->gui->tag_idle = g_idle_add((GSourceFunc)idle, lf);
		/* 進展状況を1秒毎にモニターする */
		solve_monitor_start(lf);
	}
	print_leave();
	return;
}

/** @brief 新規ゲームのダイアローグを開き、原子数と乱数の種を得る関数
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[out] natom 原子の数
 *  @param[out] seed 乱数の種
 *  @retval TRUE ダイアローグでの値が正当の場合
 *  @retval FALSE 入力不正もしくはキャンセルが押された場合
 */
static gboolean create_new_dialog(LatFoldGUI *gui, gint *natom, gint *seed) {
	GtkWidget *dialog, *w, *input_natom, *input_seed, *table, *box;
	gboolean ret;
	gboolean loop;
	print_enter();
	/* dialog */
	dialog = gtk_dialog_new_with_buttons(_("New Game"), GTK_WINDOW(gui->main),
			(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
			GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
			GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
			NULL);
	box = GTK_DIALOG(dialog)->vbox;
	/* table */
	table = w = gtk_table_new(2, 2, FALSE);
	gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	/* label1 */
	w = gtk_label_new(_("Number of Atoms"));
	gtk_table_attach_defaults(GTK_TABLE(table), w, 0, 1, 0, 1);
	gtk_widget_show(w);
	/* label2 */
	w = gtk_label_new(_("Random seed"));
	gtk_table_attach_defaults(GTK_TABLE(table), w, 0, 1, 1, 2);
	gtk_widget_show(w);
	/* input_natom */
	input_natom = w = gtk_spin_button_new_with_range(10, 60, 1);
	gtk_table_attach_defaults(GTK_TABLE(table), w, 1, 2, 0, 1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(input_natom), (gdouble)*natom);
	gtk_widget_show(w);
	/* input_seed */
	input_seed = w = gtk_spin_button_new_with_range(0.0, 4294967295.0, 1.0);
	gtk_table_attach_defaults(GTK_TABLE(table), w, 1, 2, 1, 2);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(input_seed), (gdouble)*seed);
	gtk_widget_show(w);
	do {
		switch (gtk_dialog_run(GTK_DIALOG(dialog))) {
			case GTK_RESPONSE_ACCEPT:
				*natom = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(input_natom));
				*seed = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(input_seed));
				if (*natom >= 10 && *natom <= 60) {
					ret = TRUE;
					loop = FALSE;
				} else {
					loop = TRUE;
				}
				break;
			case GTK_RESPONSE_REJECT:
			default:
				ret = FALSE;
				loop = FALSE;
				break;
		}
	} while (loop);
	gtk_widget_destroy(dialog);
	print_leave();
	return ret;
}
