#include "latfold.h"

/*! @brief LatFoldの初期化を行う。
 *  @param[in] pargc コマンドライン引数の数argcのポインター
 *  @param[in] pargv コマンドライン引数の要素argvのポインター
 *  @return 新たに作成されたLatFold変数のポインタ値。
 *  不要になったら、gfreeを行わなければならない。
 */
LatFold *latfold_new(int *pargc, char **pargv[]) {
	LatFold *lf;
	print_enter();
	lf = g_malloc(sizeof(LatFold));
	lf->data = NULL;
	latfold_option_new(lf, pargc, pargv);
	latfold_gui_new(lf);
	print_leave();
	return lf;
}

/*! @brief ユーザーからの入力待ちを行うループに入る。
 */
void latfold_loop(void) {
	print_enter();
	gtk_main();
	print_leave();
	return;
}

/*! @brief LatFold変数を解放する。
 *  @param[in] lf 解放したいLatFold変数。
 */
void latfold_destroy(LatFold *lf) {
	print_enter();
	if (lf->gui != NULL) {
		latfold_gui_destroy(lf->gui);
		lf->gui = NULL;
	}
	if (lf->data != NULL) {
		if (lf->data->solve != NULL) {
			solve_destroy(lf->data);
		}
		latfold_data_destroy(lf->data);
		lf->data = NULL;
	}
	g_nullify_pointer((gpointer)&lf);
	print_leave();
	return;
}
