#ifndef _LAT_FOLD_H_
#define _LAT_FOLD_H_

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>
#include <glib/gi18n.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif  /* HAVE_CONFIG_H */

#ifdef PRINT_FUNCTION
#define print_enter(...) (g_print("Enter %s()\n",__func__))
#define print_leave(...) (g_print("Leave %s()\n",__func__))
#else
#define print_enter(...) 
#define print_leave(...) 
#endif  /* PRINT_FUNCTION */


typedef enum {
	ATOM_NONE = 0,
	ATOM_P = 1,
	ATOM_H = 2,
	ATOM_DUMMY = 4
} AtomType;

typedef enum {
	BOND_TOP,
	BOND_BOTTOM,
	BOND_LEFT,
	BOND_RIGHT
} BondDirection;

typedef enum {
	BOND_SOLID = 0,
	BOND_LINE = 1
} BondDrawType;

typedef enum {
	NONE,
	TRANSLATION,
	BEND,
	BEND_BOND,
	FLIP1
} OperationType;

typedef enum {
	MOUSE_TYPE_NONE,
	MOUSE_TYPE_ATOM,
	MOUSE_TYPE_BOND
} MouseType;

typedef struct {
	gint x;
	gint y;
} XY;

typedef struct {
	MouseType mtype;
	OperationType otype;
	XY a1;
	XY a2;
	gint position;
} Mouse;

typedef struct {
	GtkWidget *main;
	GtkWidget *npatom;
	GtkWidget *nhatom;
	GtkWidget *score;
	GtkWidget *button_zoomin;
	GtkWidget *button_zoomout;
	GtkWidget *progress;
	GtkWidget *da;
	GtkWidget *statusbar;
	GdkPixmap *product;
	GdkPixmap *background;
	GdkPixmap *tile;
	GdkGC *gc_tile;
	GdkGC *gc_gray;
	GdkGC *gc_black;
	GdkGC *gc_blue;
	GdkGC *gc_red;
	GdkGC *gc_yellow;
	GdkGC *gc_white;
	GdkBitmap *mask[4][2];
	GdkPixmap *pixmap_bend[4];
	GdkPixmap *pixmap_flip[4];
	GdkPixmap *pixmap_arrow;
	GdkPixmap *pixmap_mouse;
	GdkPixmap *pixmap_score;
	GdkBitmap *mask_bend[4];
	GdkBitmap *mask_flip[4];
	GdkBitmap *mask_arrow;
	GdkBitmap *mask_mouse;
	GdkBitmap *mask_score;
	GtkListStore *store;
	GtkAction *action_fullscreen;
	XY pixcel_max;
	XY wc;
	gboolean translation;
	XY last;
	gboolean selected;
	XY selected_l;
	Mouse press;
	Mouse release;
	Mouse click1;
	Mouse click2;
	gboolean operatable;
	gdouble zoom;
	gint f_l2w;
	gint tilesize;
	gint bondwidth;
	gint atomradius;
	gint clist_last_score;
	gchar str_progress[7];
	guint tag_idle;
	guint tag_monitor;
	gchar *message;
	guint cid;
} LatFoldGUI ;

typedef struct {
	gint x;
	gint y;
	AtomType type;
} Atom;

typedef struct {
	gint x;
	gint y;
	gint origin;
	gint direction;
	gint rot;
} CandidateAtom;

typedef struct {
	gint x1;
	gint y1;
	gint x2;
	gint y2;
} CandidateBond;

typedef struct {
	Atom *atom;
	Atom *best;
	gint *rest_red;
	gint *direction;
	gint max_score;
	gint *space;
	gint *e_bond;
	gint *e_atom;
	GTimer *timer;
	gint ia;
	gint e_now;
	gdouble complete;
	gdouble *time;
} LatFoldDataSolve;

typedef struct {
	gint natom;
	gint nhatom;
	gint npatom;
	Atom *atom;
	Atom *work;
	gboolean *move;
	gboolean mode_preview;
	CandidateAtom *ca;
	OperationType op_type;
	gint nca;
	gint score0;
	gint seed;
	gint user_best;
	GTimer *user_timer;
	LatFoldDataSolve *solve;
} LatFoldData;

typedef struct {
	gboolean fullscreen;
	gint natom;
	gint seed;
} LatFoldOption;

typedef struct {
	LatFoldGUI *gui;
	LatFoldData *data;
	LatFoldOption *opt;
} LatFold;

LatFold *latfold_new(int *pargc, char **argv[]);
void latfold_loop(void);
void latfold_destroy(LatFold *);

void latfold_option_new(LatFold *lf, int *pargc, char **pargv[]);

void latfold_gui_new(LatFold *);
void latfold_gui_destroy(LatFoldGUI *);
void solve_monitor_start(LatFold *lf);
void set_score(LatFoldGUI *gui, gint score);
void set_npatom(LatFoldGUI *gui, gint npatom);
void set_nhatom(LatFoldGUI *gui, gint nhatom);
gboolean idle(LatFold *lf);

void make_molecule(LatFoldData *data, gint seed);
LatFoldData *latfold_data_new(LatFoldGUI *gui, gint natom, gint seed);
void latfold_data_destroy(LatFoldData *data);
gint lattice2atom (Atom *atom, gint natom, XY *l);
gint lattice2bond (Atom *atom, gint natom, XY *l1, XY *l2);
gint candidate_place(CandidateAtom *ca, gint nca, Mouse *mouse);

void lattice2widget(LatFoldGUI *gui, XY *l, XY *w);
void widget2lattice(LatFoldGUI *gui, XY *w, XY *l);
gint iround(gdouble x);
void lattice_position(LatFold *lf, XY w, XY *l, gint *position);
gint set_candidate_atom(LatFoldData *data, Mouse *mouse);
void clear_candidate_atom(LatFoldData *data);
void move_atom(LatFoldData *data, Mouse *mouse, gboolean preview);
gint set_candidate_bond_bend(LatFoldData *data, Mouse *mouse);
void move_bond_bend(LatFoldData *data, Mouse *mouse, gboolean preview);
gint set_candidate_flip1(LatFoldData *data, Mouse *mouse);
void move_atom_flip1(LatFoldData *data, Mouse *mouse, gboolean preview);
gint calc_score(LatFoldData *data, Atom *atom);

GtkWidget *latfold_gui_da_new(LatFold *lf);
void draw(LatFold *lf);
void create_da_work(LatFoldGUI *gui);
void create_tile(LatFoldGUI *gui);
void create_background(LatFoldGUI *gui);
void create_gc(LatFoldGUI *gui);
void create_mask(LatFoldGUI *gui);

GtkWidget *create_menu(LatFold *lf);

void set_statusbar(LatFoldGUI *gui, gchar *mesg);
void clear_statusbar(LatFoldGUI *gui);

void set_icon(void);

GtkWidget *latfold_gui_clist_new(LatFoldGUI *gui);
void latfold_gui_clist_set_score(LatFoldGUI *gui, gint score, gchar *str_time, gboolean overwrite);
void latfold_gui_clist_clear(LatFoldGUI *gui);

void open_new_dialog(LatFold *lf);

void load_help_pixmap(LatFoldGUI *gui);
void help_dialog(GtkAction *action, LatFold *lf);

void open_about_dialog(void);

void solve_init(LatFoldData *data);
gboolean solve(LatFoldData *data);
void solve_destroy(LatFoldData *data);

#endif /* _LAT_FOLD_H_ */
