#include "latfold.h"

/**! @brief LatFoldOptionの初期化を行う関数
 *  @param[in] lf LatFoldのポインター
 *  @param[in] pargc コマンドライン引数の数argcのポインター
 *  @param[in] pargv コマンドライン引数の要素argvのポインター
 */
void latfold_option_new(LatFold *lf, int *pargc, char **pargv[]) {
	static gboolean fullscreen = FALSE;
	static gint natom = 10;
	static gint seed;
	static GOptionEntry entries[] =
	{
		{ "fullscreen", 0, 0, G_OPTION_ARG_NONE, &fullscreen, N_("Fullscreen Mode"), NULL},
		{ "natom", 0, 0, G_OPTION_ARG_INT, &natom, N_("Number of Atoms"), "N"},
		{ "seed", 0, 0, G_OPTION_ARG_INT, &seed, N_("Random Seed"), "S"},
		{ NULL }
	};
	GError *error = NULL;
	GOptionContext *context;
	GRand *rand;
	rand = g_rand_new();
	seed = g_rand_int_range(rand, 0, 2147483647);
	context = g_option_context_new(_("- game for folding of lattice model"));
	g_option_context_add_main_entries(context, entries, NULL);
	g_option_context_add_group(context, gtk_get_option_group(TRUE));
	if (!g_option_context_parse(context, pargc, pargv, &error)) {
		g_print("Error in parsing: %s\n", error->message);
		exit(1);
	}
	lf->opt = g_malloc(sizeof(LatFoldOption));
	lf->opt->fullscreen = fullscreen;
	lf->opt->natom = natom;
	lf->opt->seed = seed;
	return;
}
