#include "latfold.h"
#include <string.h>

#define pos(x,y) ((((x+1)+natom))*(natom*2+2)+(((y+1)+natom)))

#undef PROFILE

#ifdef PROFILE
static GTimer *t_init = NULL;
static GTimer *t_score = NULL;
static GTimer *t_erase = NULL;
static GTimer *t_next = NULL;
static gint n_init = 0;
static gint n_score = 0;
static gint n_erase = 0;
static gint n_next = 0;
#endif

/** @brief 探索ルーチンの初期化を行う関数
 *  @param[in] data LatFoldDataのポインター
 */
void solve_init(LatFoldData *data) {
	LatFoldDataSolve *solve;
	Atom *atom;
	gint natom;
	gint i, j;
	gint prev_type;
	gint *space;
	gint *e_bond;
	natom = data->natom;
	/* 原子数が1個のときは即終了する */
	if (natom < 2) {
		return;
	}
	print_enter();
	data->solve = solve = g_malloc(sizeof(LatFoldDataSolve));
	solve->atom = atom = g_malloc(sizeof(Atom)*natom);
	solve->best = g_malloc(sizeof(Atom)*natom);
	solve->rest_red = g_malloc(sizeof(gint)*natom);
	solve->direction = g_malloc(sizeof(gint)*natom);
	solve->space = space = g_malloc(sizeof(gint)*(natom*2+2)*(natom*2+2));
	solve->e_bond = e_bond = g_malloc0(sizeof(gint)*natom);
	solve->e_atom = g_malloc0(sizeof(gint)*natom);
	solve->max_score = 0;
	solve->timer = g_timer_new();
	solve->ia = 1;
	solve->e_now = 0;
	solve->complete = 0.0;
	solve->time = NULL;
	memcpy(solve->atom, data->atom, sizeof(Atom)*natom);
	memcpy(solve->best, data->atom, sizeof(Atom)*natom); /* 最適解が0のとき用 */
	/* 赤の残りの個数が何個かを登録する */
	if (atom[natom-1].type == ATOM_H) {
		solve->rest_red[natom-1] = 1;
	} else {
		solve->rest_red[natom-1] = 0;
	}
	for (i = natom-2; i >= 0; --i) {
		if (atom[i].type == ATOM_H) {
			solve->rest_red[i] = solve->rest_red[i+1] + 1;
		} else {
			solve->rest_red[i] = solve->rest_red[i+1];
		}
	}
	/* 一番目の原子は原点に置く */
	atom[0].x = 0;
	atom[0].y = 0;
	/* 二番目以降の原子は全探索する */
	/* e_bondの初期化 */
	prev_type = -1;
	for (j = 0; j < natom; ++j) {
		if (prev_type == ATOM_H && data->atom[j].type == ATOM_H) {
			e_bond[j] = 1;
		} else {
			e_bond[j] = 0;
		}
		prev_type = data->atom[j].type;
	}
	/* 全探索用に格子空間の作業領域をとり、0番の原子を配置する */
	for (j = 0; j < (natom*2+2)*(natom*2+2); ++j) {
		space[j] = -1;
	}
	space[pos(atom[0].x,atom[0].y)] = 0;
	/* 探索方向の初期化 */
	solve->direction[0] = 0;
	solve->direction[1] = 0;
#ifdef PROFILE
	t_init = g_timer_new();
	g_timer_stop(t_init);
	t_score = g_timer_new();
	g_timer_stop(t_score);
	t_erase = g_timer_new();
	g_timer_stop(t_erase);
	t_next = g_timer_new();
	g_timer_stop(t_next);
#endif
	print_leave();
	return;
}

/** @brief 探索を行う関数
 *  @param[in] data LatFoldDataのポインター
 *  @retval TRUE 探索未完了
 *  @retval FALSE 探索完了
 */
gboolean solve(LatFoldData *data) {
	LatFoldDataSolve *solve;
	Atom *atom;
	gint natom;
	gint i, j, k, ia, last_x, last_y, now_x, now_y, e_now, pos_now;
	gint isearch = 0;
	gint dx[] = {1,0,-1,0};
	gint dy[] = {0,1,0,-1};
	gint d_pos[4];
	gint *space;
	gint *e_bond;
	gint *e_atom;
	gboolean finish = TRUE;
	/* 探索用のデータがない場合はすぐに抜ける */
	if (data == NULL || data->solve == NULL) {
		return FALSE;
	}
	print_enter();
	natom = data->natom;
	for (i = 0; i < 4; ++i) {
		d_pos[i] = pos(dx[i],dy[i]) - pos(0,0);
	}
	solve = data->solve;
	atom = solve->atom;
	space = solve->space;
	e_bond = solve->e_bond;
	e_atom = solve->e_atom;
	ia = solve->ia;  /* ポインターでないので最後に書き戻しを忘れずに */
	e_now = solve->e_now;  /* ポインターでないので最後に書き戻しを忘れずに */
	/* 探索を最大10000回行い、一旦GTKのメインループに制御を返す */
#ifdef PROFILE
	for (isearch = 0; solve->direction[1] < 1 && isearch < 100000; ++isearch) {
#else  /* PROFILE */
	for (isearch = 0; solve->direction[1] < 1 && isearch < 10000; ++isearch) {
#endif  /* PROFILE */
		gboolean samelevel = FALSE;
#ifdef PROFILE
		g_timer_continue(t_init);
		++n_init;
#endif
		/* 今の階層よりも下の階層を初期化する */
		for (j = ia+1; j < natom; ++j) {
			solve->direction[j] = 0;
		}
#if 0
		g_print("level: %d  :", ia);
		for (j = 0; j <= ia; ++j) {
			g_print(" %d", solve->direction[j]);
		}
		g_print("\n");
#endif
		last_x = atom[ia-1].x;
		last_y = atom[ia-1].y;
		i = solve->direction[ia];
		/* ia番目の原子を配置する */
		now_x = atom[ia].x = last_x + dx[i];
		now_y = atom[ia].y = last_y + dy[i];
		/* 新たに置こうとした原子がこれまでの原子と重なりがあるか */
		pos_now = pos(now_x,now_y);
#ifdef PROFILE
		g_timer_stop(t_init);
#endif
		if (space[pos_now] >= 0) {
			/* 新たに配置した原子が既にある原子と重なった場合 */
			samelevel = TRUE;
		} else {
#ifdef PROFILE
			g_timer_continue(t_score);
			++n_score;
#endif
			/* spaceに登録する */
			space[pos_now] = ia;
			e_atom[ia] = -e_bond[ia];
			/* 新たに配置した原子に関して得点を計算する */
			if (atom[ia].type == ATOM_H) {
				for (k = 0; k < 4; ++k) {
					gint sk;
					sk = space[pos_now+d_pos[k]];
					if (sk >= 0 && atom[sk].type == ATOM_H) {
						++e_atom[ia];
					}
				}
			}
			/* 全原子の得点を差分で計算する。
			 */
			e_now += e_atom[ia];
#ifdef PROFILE
			g_timer_stop(t_score);
#endif
			/* 最後の原子で、最高得点なら、記録する。
			 * 残りの赤の個数*2が残りの最大加点であるが、記録更新できそうなら、
			 * 次の原子を配置するためにこの関数を再帰呼び出しする。
			 * 最後の一個は3個の結合が可能
			 */
			if (ia >= natom - 1) {
				if (e_now > solve->max_score) {
					gint l;
					solve->time = g_realloc(solve->time, sizeof(gdouble)*e_now);
					for (l = solve->max_score; l < e_now; ++l) {
						solve->time[l] = g_timer_elapsed(solve->timer, NULL);
					}
					solve->max_score = e_now;
					memcpy(solve->best, solve->atom, sizeof(Atom)*natom);
				}
				/* 末端に達したので、spaceから登録を削除する */
				space[pos_now] = -1;
				/* e_nowを差分を使って戻す */
				e_now -= e_atom[ia];
				samelevel = TRUE;
			} else if (e_now + solve->rest_red[ia]*2 + solve->rest_red[natom-1] <= solve->max_score) {
#ifdef PROFILE
				g_timer_continue(t_erase);
				++n_erase;
#endif
				/* 得点更新の見込みがないので、spaceから登録を削除する */
				space[pos_now] = -1;
				/* e_nowを差分を使って戻す */
				e_now -= e_atom[ia];
				samelevel = TRUE;
#ifdef PROFILE
				g_timer_stop(t_erase);
#endif
			}
		}
#ifdef PROFILE
		g_timer_continue(t_next);
		++n_next;
#endif
		if (samelevel) {
			/* 今の階層でdirectionを1増やす */
			++solve->direction[ia];
			/* 繰り上がりを計算する。 */
			while ((ia >= 3 && solve->direction[ia] >= 4)
					|| (ia == 2 && solve->direction[ia] >= 2)
					|| (ia == 1 && solve->direction[ia] >= 1)) {
				++solve->direction[--ia];
				space[pos(atom[ia].x,atom[ia].y)] = -1;
				e_now -= e_atom[ia];
				if (ia == 5) {
					solve->complete = (solve->direction[2] / 2.0 + solve->direction[3] / 8.0 + solve->direction[4] / 32.0 + solve->direction[5]/128.0); 
				}
			}
		} else {
			/* 階層を一つ降りる */
			++ia;
		}
#ifdef PROFILE
		g_timer_stop(t_next);
#endif
	}
#ifdef PROFILE
	g_print("init=%.2f/%d score=%.2f/%d erase=%.2f/%d next=%.2f/%d\n", 
			g_timer_elapsed(t_init, NULL), n_init,
			g_timer_elapsed(t_score, NULL), n_score,
			g_timer_elapsed(t_erase, NULL), n_erase,
			g_timer_elapsed(t_next, NULL), n_next); 
#endif
	solve->ia = ia;  /* 次回の探索のための書き戻し */
	solve->e_now = e_now;  /* 次回の探索のための書き戻し */
	if (solve->direction[1] >= 1) {
		solve->complete = 1.0;
		finish = FALSE;
	}
	print_leave();
	return finish;
}

/** @brief 探索の後処理を行う関数
 *  @param[in] data LatFoldDataのポインター
 */
void solve_destroy(LatFoldData *data) {
	LatFoldDataSolve *solve;
	/* 探索用のデータがない場合はすぐに抜ける */
	if (data == NULL || data->solve == NULL) {
		return;
	}
	print_enter();
	solve = data->solve;
	/* 探索用メモリーの解放 */
	g_timer_destroy(solve->timer);
	solve->timer = NULL;
	g_nullify_pointer((gpointer)&solve->atom);
	g_nullify_pointer((gpointer)&solve->best);
	g_nullify_pointer((gpointer)&solve->rest_red);
	g_nullify_pointer((gpointer)&solve->space);
	g_nullify_pointer((gpointer)&solve->e_bond);
	g_nullify_pointer((gpointer)&solve->e_atom);
	g_nullify_pointer((gpointer)&solve->direction);
	g_nullify_pointer((gpointer)&solve->time);
	g_nullify_pointer((gpointer)&data->solve);
	print_leave();
	return;
}

