#include "latfold.h"

/** @brief lattice座標系をwidget座標系に変換
 *  @param[in] gui LatFoldGUIポインター値
 *  @param[in] l lattice座標
 *  @param[out] w widget座標
 */
void lattice2widget(LatFoldGUI *gui, XY *l, XY *w) {
	print_enter();
	w->x = l->x * gui->tilesize - gui->wc.x + gui->pixcel_max.x/2;
	w->y = l->y * gui->tilesize - gui->wc.y + gui->pixcel_max.y/2;
	print_leave();
	return;
}

/** @brief widget座標系をlattice座標系に変換
 *  @param[in] gui LatFoldGUIポインター値
 *  @param[in] w widget座標
 *  @param[out] l lattice座標
 */
void widget2lattice(LatFoldGUI *gui, XY *w, XY *l) {
	print_enter();
	l->x = iround((w->x - gui->pixcel_max.x/2 + gui->wc.x) / (double)gui->tilesize);
	l->y = iround((w->y - gui->pixcel_max.y/2 + gui->wc.y) / (double)gui->tilesize);
	print_leave();
	return;
}

/** @brief 倍精度値を最も近い整数値に丸める関数
 *  @param[in] x 丸められる倍精度値
 *  @return 丸められた整数値
 */
gint iround(gdouble x) {
	gint i;
	print_enter();
	if (x > 0.0) {
		i = (gint)(x + 0.5);
	} else {
		i = (gint)(x - 0.5);
	}
	print_leave();
	return i;
}

/** @brief widget座標系の座標がどのlatticeのどのパーツかを返す関数
 *  @param[in] lf LatFoldポインター
 *  @param[in] w Widget座標
 *  @param[out] l Latticeの場所の座標
 *  @param[out] position Latticeから見た相対的場所(9分割)
 */
void lattice_position(LatFold *lf, XY w, XY *l, gint *position) {
	LatFoldGUI *gui;
	XY wa, rw;
	gdouble r, hw;
	print_enter();
	gui = lf->gui;
	widget2lattice(gui, &w, l);
	lattice2widget(gui, l, &wa);
	rw.x = w.x - wa.x;
	rw.y = w.y - wa.y;
	r = iround(gui->atomradius*gui->zoom);
	hw = iround(gui->bondwidth/2*gui->zoom);
	if (rw.x * rw.x + rw.y * rw.y <= r * r) {
		*position = 5;
	} else if (rw.x >= -hw && rw.x <= hw) {
		if (rw.y > 0) {
			*position = 2;
		} else {
			*position = 8;
		}
	} else if (rw.y >= -hw && rw.y <= hw) {
		if (rw.x > 0) {
			*position = 6;
		} else {
			*position = 4;
		}
	} else if (rw.x > 0.0) {
		if (rw.y > 0) {
			*position = 3;
		} else {
			*position = 9;
		}
	} else {
		if (rw.y > 0) {
			*position = 1;
		} else {
			*position = 7;
		}
	}
	print_leave();
	return;
}
