#include "latfold.h"
#include "icon/hydrophobic.xpm"
#include "icon/polar.xpm"

static void zoomin(GtkButton *button, LatFold *lf);
static void zoomout(GtkButton *button, LatFold *lf);
static void zoom(LatFold *lf, gdouble scale);
static gboolean solve_monitor(LatFold *lf);
static void set_integer_in_label(GtkWidget *widget, gint i);

/** @brief 拡大ボタンが押されたときのコールバック関数
 *  @param[in] button 拡大ボタンwidget
 *  @param[in] lf LatFoldのポインター
 */
static void zoomin(GtkButton *button, LatFold *lf) {
	print_enter();
	g_return_if_fail(button != NULL);
	zoom(lf, 1.25);
	print_leave();
	return;
}

/** @brief 縮小ボタンが押されたときのコールバック関数
 *  @param[in] button 縮小ボタンwidget
 *  @param[in] lf LatFoldのポインター
 */
static void zoomout(GtkButton *button, LatFold *lf) {
	print_enter();
	g_return_if_fail(button != NULL);
	zoom(lf, 0.8);
	print_leave();
	return;
}

/** @brief 拡大縮小を行う関数
 */
static void zoom(LatFold *lf, gdouble scale) {
	LatFoldGUI *gui;
	gint new_tilesize;
	gdouble new_zoom;
	print_enter();
	gui = lf->gui;
	new_tilesize = iround(gui->tilesize * scale);
	new_zoom = new_tilesize / (double)gui->f_l2w;
	if (new_zoom < 2.0 && new_zoom > 0.2) {
		gui->tilesize = new_tilesize;
	 	gui->zoom = new_zoom;
		create_mask(gui);
		create_tile(gui);
		create_background(gui);
		draw(lf);
	}
	print_leave();
	return;
}

/** @brief LatFoldGUIの初期化を行う。
 *  @param[in] lf LatFoldのポインター
 *  @return 新たに作成されたLatFoldGUI変数のポインタ値。
 *  不要になったら、gfreeを行わなければならない。
 */
void latfold_gui_new(LatFold *lf) {
	LatFoldGUI *gui;
	GtkWidget *w, *hbox_left, *hbox, *vbox_left, *vbox_menu_body, *sw, *c;
	GtkWidget *hbox_p, *hbox_h, *vbox_right;
	GdkPixbuf *p;
	gint i, j;
	print_enter();
	lf->gui = gui = g_malloc(sizeof(LatFoldGUI));
	gui->bondwidth = 30;
	gui->atomradius = 27;
	gui->background = NULL;
	gui->product = NULL;
	gui->tile = NULL;
	gui->action_fullscreen = NULL;
	gui->wc.x = 0;
	gui->wc.y = 0;
	gui->f_l2w = 100;
	gui->translation = FALSE;
	gui->selected = FALSE;
	gui->press.mtype = MOUSE_TYPE_NONE;
	gui->click1.mtype = MOUSE_TYPE_NONE;
	gui->click2.mtype = MOUSE_TYPE_NONE;
	gui->press.otype = NONE;
	gui->click1.otype = NONE;
	gui->click2.otype = NONE;
	gui->operatable = TRUE;
	gui->tilesize = 33;
	gui->zoom = gui->tilesize / (double)gui->f_l2w;
	gui->clist_last_score = 0;
	gui->tag_idle = 0;
	gui->tag_monitor = 0;
	for (i = 0; i < 4; ++i) {
		for (j = 0; j < 2; ++j) {
			gui->mask[i][j] = NULL;
		}
		gui->pixmap_bend[i] = NULL;
		gui->pixmap_flip[i] = NULL;
		gui->mask_bend[i] = NULL;
		gui->mask_flip[i] = NULL;
	}
	gui->pixmap_arrow = NULL;
	gui->pixmap_mouse = NULL;
	gui->pixmap_score = NULL;
	gui->mask_arrow = NULL;
	gui->mask_mouse = NULL;
	gui->mask_score = NULL;
	/* アイコンを設定する */
	set_icon();
	/* main window */
	gui->main = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_resizable(GTK_WINDOW(gui->main), TRUE);
	gtk_window_set_default_size(GTK_WINDOW(gui->main), 700, 700);
	g_signal_connect(G_OBJECT(gui->main), "destroy",
			G_CALLBACK(gtk_main_quit), NULL);
	gtk_widget_realize(gui->main);
	/* vbox_menu_body */
	vbox_menu_body = w = gtk_vbox_new(FALSE, FALSE);
	gtk_container_add(GTK_CONTAINER(gui->main), w);
	gtk_widget_show(w);
	/* menu */
	w = create_menu(lf);
	gtk_box_pack_start(GTK_BOX(vbox_menu_body), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	/* hbox */
	hbox = w = gtk_hbox_new(FALSE, FALSE);
	gtk_box_pack_start(GTK_BOX(vbox_menu_body), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	/* vbox_left */
	vbox_left = w = gtk_vbox_new(FALSE, FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	/* hbox_left */
	hbox_left = w = gtk_hbox_new(FALSE, FALSE);
	gtk_box_pack_start(GTK_BOX(vbox_left), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	/* frame_h */
	c = w = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(hbox_left), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	/* hbox_h */
	hbox_h = w = gtk_hbox_new(FALSE, FALSE);
	gtk_container_add(GTK_CONTAINER(c), w);
	gtk_widget_show(w);
	/* image_h */
	p = gdk_pixbuf_new_from_xpm_data((const char **)hydrophobic_xpm);
	w = gtk_image_new_from_pixbuf(p);
	gtk_box_pack_start(GTK_BOX(hbox_h), w, TRUE, FALSE, 0);
	gtk_widget_show(w);
	g_object_unref(p);
	/* nhatom */
	gui->nhatom = w = gtk_label_new("0");
	gtk_box_pack_start(GTK_BOX(hbox_h), w, TRUE, FALSE, 0);
	gtk_widget_show(w);
	/* frame_p */
	c = w = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(hbox_left), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	/* hbox_p */
	hbox_p = w = gtk_hbox_new(FALSE, FALSE);
	gtk_container_add(GTK_CONTAINER(c), w);
	gtk_widget_show(w);
	/* image_p */
	p = gdk_pixbuf_new_from_xpm_data((const char **)polar_xpm);
	w = gtk_image_new_from_pixbuf(p);
	gtk_box_pack_start(GTK_BOX(hbox_p), w, TRUE, FALSE, 0);
	gtk_widget_show(w);
	g_object_unref(p);
	/* npatom */
	gui->npatom = w = gtk_label_new("0");
	gtk_box_pack_start(GTK_BOX(hbox_p), w, TRUE, FALSE, 0);
	gtk_widget_show(w);
	/* label_score */
	w = gtk_label_new(_("Score"));
	gtk_box_pack_start(GTK_BOX(vbox_left), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	/* score */
	gui->score = w = gtk_label_new("0");
	gtk_box_pack_start(GTK_BOX(vbox_left), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	/* button_zoomin */
	gui->button_zoomin = w = gtk_button_new();
	gtk_box_pack_start(GTK_BOX(vbox_left), w, FALSE, FALSE, 0);
	g_signal_connect(G_OBJECT(w), "clicked",
			G_CALLBACK(zoomin), lf);
	gtk_widget_show(w);
	/* image_zoomin */
	w = gtk_image_new_from_stock(GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_BUTTON);
	gtk_button_set_image(GTK_BUTTON(gui->button_zoomin), w);
	gtk_widget_show(w);
	/* button_zoomout */
	gui->button_zoomout = w = gtk_button_new();
	gtk_box_pack_start(GTK_BOX(vbox_left), w, FALSE, FALSE, 0);
	g_signal_connect(G_OBJECT(w), "clicked",
			G_CALLBACK(zoomout), lf);
	gtk_widget_show(w);
	/* image_zoomout */
	w = gtk_image_new_from_stock(GTK_STOCK_ZOOM_OUT, GTK_ICON_SIZE_BUTTON);
	gtk_button_set_image(GTK_BUTTON(gui->button_zoomout), w);
	gtk_widget_show(w);
	/* scrolled_window */
	sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
			GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(vbox_left), sw, TRUE, TRUE, 0);
	gtk_widget_show(sw);
	/* clist */
	w = latfold_gui_clist_new(gui);
	gtk_container_add(GTK_CONTAINER(sw), w);
	gtk_widget_show(w);
	/* progress */
	gui->progress = w = gtk_progress_bar_new();
	gtk_progress_bar_set_pulse_step(GTK_PROGRESS_BAR(w), 0.01);
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(w), 0.0);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(w), NULL);
	gtk_box_pack_start(GTK_BOX(vbox_left), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	/* vbox_right */
	vbox_right = w = gtk_vbox_new(FALSE, FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	/* Drawing Area */
	gui->da = w = latfold_gui_da_new(lf);
	gtk_box_pack_start(GTK_BOX(vbox_right), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	/* statusbar */
	gui->statusbar = w = gtk_statusbar_new();
	gtk_box_pack_start(GTK_BOX(vbox_right), w, FALSE, FALSE, 0);
	set_statusbar(gui, _("To start, select \"New\" from \"Game\" menu."));
	gtk_widget_show(w);
	/* Create gc and set foreground color */
	create_gc(gui);
	/* フルスクリーンモードにするには、
	 * メニューからフルスクリーンを選んだことにする。 */
	if (lf->opt->fullscreen) {
		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(gui->action_fullscreen), TRUE);
		gtk_toggle_action_toggled(GTK_TOGGLE_ACTION(gui->action_fullscreen));
	}
	/* ヘルプ用画像をロードする */
	load_help_pixmap(lf->gui);
	/* Finally show toplevel window */
	gtk_widget_show(gui->main);
	/* 初期状態はまだ分子を読み込んでいないので、まだ描くことはしない */
	print_leave();
	return;
}

/** @brief 探索の進展状況をモニターする関数を起動する関数
 *  @param lf LatFoldのポインター
 */
void solve_monitor_start(LatFold *lf) {
	LatFoldGUI *gui = lf->gui;
	if (gui->tag_monitor > 0) {
		g_source_remove(gui->tag_monitor);
		gui->tag_monitor = 0;
	}
#if GTK_CHECK_VERSION(2,14,0)  /* gtk+-2.0 >= 2.14.0 */
	gui->tag_monitor = g_timeout_add_seconds(1, (GSourceFunc)solve_monitor, lf);
#else  /* gtk+-2.0 < 2.14.0 */
	gui->tag_monitor = g_timeout_add(1000, (GSourceFunc)solve_monitor, lf);
#endif
	return;
}

/** @brief 探索の進展状況を一秒毎にモニターする関数
 *  @param lf LatFoldのポインター
 *  @retval TRUE 探索中の場合
 *  @retval FALSE 探索終了の場合
 */
gboolean solve_monitor(LatFold *lf) {
	gboolean ret;
	if (lf->data != NULL && lf->data->solve != NULL) {
		GtkProgressBar *pb;
		gint i;
		pb = GTK_PROGRESS_BAR(lf->gui->progress);
		if (lf->data->solve->complete < 0.999) {
			/* 探索中はプログレスバーを正確に表示する */
			sprintf(lf->gui->str_progress, "%.1f%%", lf->data->solve->complete*100);
			gtk_progress_bar_set_text(pb, lf->gui->str_progress);
			gtk_progress_bar_set_fraction(pb, lf->data->solve->complete);
			ret = TRUE;
		} else {
			/* 探索していないときはプログレスバーを0に戻す */
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(lf->gui->progress), 0.0);
			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(lf->gui->progress), NULL);
			set_statusbar(lf->gui, _("Comupter found the best solution."));
			lf->gui->tag_monitor = 0;
			ret = FALSE;
		}
		for (i = lf->gui->clist_last_score; i < lf->data->solve->max_score; ++i) {
			gchar *str;
			str = g_strdup_printf("(%.4f)", lf->data->solve->time[i]);
			latfold_gui_clist_set_score(lf->gui, i+1, str, FALSE);
			g_nullify_pointer((gpointer)&str);
		}
		lf->gui->clist_last_score = lf->data->solve->max_score;
	}
	return ret;
}

/** @brief LatFoldGUI変数を解放する。
 *  @param[in] gui 解放したいLatFoldGUI変数。
 */
void latfold_gui_destroy(LatFoldGUI *gui) {
	print_enter();
	gint i;
	for (i = 0; i < 4; ++i) {
		g_object_unref(gui->pixmap_bend[i]);
		g_object_unref(gui->pixmap_flip[i]);
		g_object_unref(gui->mask_bend[i]);
		g_object_unref(gui->mask_flip[i]);
	}
	g_object_unref(gui->pixmap_mouse);
	g_object_unref(gui->pixmap_arrow);
	g_object_unref(gui->pixmap_score);
	g_object_unref(gui->mask_mouse);
	g_object_unref(gui->mask_arrow);
	g_object_unref(gui->mask_score);
	g_nullify_pointer((gpointer)&gui);
	print_leave();
	return;
}

/** @brief labelウィジェットに整数を表示させる関数
 *  @param[in] widget labelウィジェットのGtkWidgetポインター
 *  @param[in] i 表示させたい整数値
 */
static void set_integer_in_label(GtkWidget *widget, gint i) {
	gchar *str;
	str = g_strdup_printf("%d", i);
	gtk_label_set_text(GTK_LABEL(widget), str);
	g_nullify_pointer((gpointer)&str);
	return;
}

/** @brief 得点を表示させる関数
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[in] score 得点
 */
void set_score(LatFoldGUI *gui, gint score) {
	print_enter();
	set_integer_in_label(gui->score, score);
	print_leave();
	return;
}

/** @brief 親水性(hydorophilic, ie., polar)原子数を表示させる関数
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[in] npatom P原子の数
 */
void set_npatom(LatFoldGUI *gui, gint npatom) {
	print_enter();
	set_integer_in_label(gui->npatom, npatom);
	print_leave();
	return;
}

/** @brief 疎水性(hydrophobic, ie., nonpolar)原子数を表示させる関数
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[in] nhatom P原子の数
 */
void set_nhatom(LatFoldGUI *gui, gint nhatom) {
	print_enter();
	set_integer_in_label(gui->nhatom, nhatom);
	print_leave();
	return;
}

/** @brief 探索を開始後、空いている時間に呼び出される関数 
 *  @param[in] lf LatFoldのポインター
 *  @retval TRUE  探索を続ける場合
 *  @retval FALSE 探索が終了した場合
 */
gboolean idle(LatFold *lf) {
	gboolean ret = FALSE;
	if (lf->data != NULL && lf->data->solve != NULL) {
		ret = solve(lf->data);
	}
	return ret;
}
